@@ -141,6 +141,12 @@ def setup_server():
         def index(self):
             raise cherrypy.InternalRedirect("/")
         
+        def relative(self):
+            raise cherrypy.InternalRedirect("cousin")
+        
+        def cousin(self):
+            return "I am a redirected page."
+        
         def petshop(self, user_id):
             if user_id == "parrot":
                 # Trade it for a slug when redirecting
@@ -219,9 +225,8 @@ def setup_server():
 
     class Ranges(Test):
         
-        def get_ranges(self):
-            h = cherrypy.request.headers.get('Range')
-            return repr(httptools.getRanges(h, 8))
+        def get_ranges(self, bytes):
+            return repr(httptools.getRanges('bytes=%s' % bytes, 8))
         
         def slice_file(self):
             path = os.path.join(os.getcwd(), os.path.dirname(__file__))
@@ -237,6 +242,9 @@ def setup_server():
             raise cherrypy.HTTPError(417, 'Expectation Failed')
 
     class Headers(Test):
+        def default(self, headername): 
+            """Spit back out the value for the requested header.""" 
+            return cherrypy.request.headers[headername]
         
         def doubledheaders(self):
             # From http://www.cherrypy.org/ticket/165:
@@ -257,8 +265,13 @@ def setup_server():
             hMap['Expires'] = 'Thu, 01 Dec 2194 16:00:00 GMT'
             
             return "double header test"
-
-
+        
+        def ifmatch(self):
+            val = cherrypy.request.headers['If-Match']
+            cherrypy.response.headers['ETag'] = val
+            return repr(val)
+    
+    
     class HeaderElements(Test):
         
         def get_elements(self, headername):
@@ -301,7 +314,7 @@ def setup_server():
         def index(self):
             yield "<h1>Choose your document</h1>\n"
             yield "<ul>\n"
-            for id, contents in self.documents:
+            for id, contents in self.documents.iteritems():
                 yield ("    <li><a href='/divorce/get?ID=%s'>%s</a>: %s</li>\n"
                        % (id, id, contents))
             yield "</ul>"
@@ -425,7 +438,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         
         self.getPage("/status/illegal")
         self.assertStatus(500)
-        msg = "Illegal response status from server (out of range)."
+        msg = "Illegal response status from server (781 is out of range)."
         self.assertErrorPage(500, msg)
         
         self.getPage("/status/unknown")
@@ -434,7 +447,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         
         self.getPage("/status/bad")
         self.assertStatus(500)
-        msg = "Illegal response status from server (non-numeric)."
+        msg = "Illegal response status from server ('error' is non-numeric)."
         self.assertErrorPage(500, msg)
     
     def testLogging(self):
@@ -570,6 +583,11 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.getPage("/internalredirect/secure")
         self.assertBody('Please log in')
         self.assertStatus(200)
+        self.assertStatus(200)
+        
+        # Relative path in InternalRedirect
+        self.getPage("/internalredirect/relative")
+        self.assertBody('I am a redirected page.')
         
         # HTTPRedirect on error
         self.getPage("/redirect/error/")
@@ -580,9 +598,11 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.getPage("/redirect/stringify", protocol="HTTP/1.0")
         self.assertStatus(200)
         self.assertBody("(['http://127.0.0.1:%s/'], 302)" % self.PORT)
-        self.getPage("/redirect/stringify", protocol="HTTP/1.1")
-        self.assertStatus(200)
-        self.assertBody("(['http://127.0.0.1:%s/'], 303)" % self.PORT)
+        
+        if cherrypy.config.get('server.socket_protocol') == 'HTTP/1.1':
+            self.getPage("/redirect/stringify", protocol="HTTP/1.1")
+            self.assertStatus(200)
+            self.assertBody("(['http://127.0.0.1:%s/'], 303)" % self.PORT)
     
     def testFlatten(self):
         for url in ["/flatten/as_string", "/flatten/as_list",
@@ -614,7 +634,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             
             # No traceback should be present
             self.getPage("/error/cause_err_in_finalize")
-            msg = "Illegal response status from server (non-numeric)."
+            msg = "Illegal response status from server ('ZOO' is non-numeric)."
             self.assertErrorPage(500, msg, None)
         finally:
             ignore.pop()
@@ -639,11 +659,15 @@ class CoreRequestHandlingTest(helper.CPWebCase):
 ##        self.assertBody("THROWN ERROR: ValueError")
     
     def testRanges(self):
-        self.getPage("/ranges/get_ranges", [('Range', 'bytes=3-6')])
+        if cherrypy.config.get('server.socket_protocol') != 'HTTP/1.1':
+            print "skipped ",
+            return
+        
+        self.getPage("/ranges/get_ranges?bytes=3-6")
         self.assertBody("[(3, 7)]")
         
         # Test multiple ranges and a suffix-byte-range-spec, for good measure.
-        self.getPage("/ranges/get_ranges", [('Range', 'bytes=2-4,-1')])
+        self.getPage("/ranges/get_ranges?bytes=2-4,-1")
         self.assertBody("[(2, 5), (7, 8)]")
         
         # Get a partial file.
@@ -664,17 +688,17 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         expected_type = "multipart/byteranges; boundary="
         self.assert_(ct.startswith(expected_type))
         boundary = ct[len(expected_type):]
-        expected_body = """--%s
-Content-type: text/html
-Content-range: bytes 4-6/14
-
-o, w
---%s
-Content-type: text/html
-Content-range: bytes 2-5/14
-
-llo, 
---%s""" % (boundary, boundary, boundary)
+        expected_body = ("\r\n--%s\r\n"
+                         "Content-type: text/html\r\n"
+                         "Content-range: bytes 4-6/14\r\n"
+                         "\r\n"
+                         "o, \r\n"
+                         "--%s\r\n"
+                         "Content-type: text/html\r\n"
+                         "Content-range: bytes 2-5/14\r\n"
+                         "\r\n"
+                         "llo,\r\n"
+                         "--%s--\r\n" % (boundary, boundary, boundary))
         self.assertBody(expected_body)
         self.assertHeader("Content-Length")
         
@@ -743,7 +767,14 @@ llo,
         self.assertBody("da\n"
                         "en-gb;q=0.8\n"
                         "en;q=0.7")
-    
+        
+        # Test that two request headers are collapsed into one.
+        # See http://www.cherrypy.org/ticket/542.
+        self.getPage("/headers/Accept-Charset",
+                     headers=[("Accept-Charset", "iso-8859-5"),
+                              ("Accept-Charset", "unicode-1-1;q=0.8")])
+        self.assertBody("iso-8859-5, unicode-1-1;q=0.8")
+        
     def testHeaders(self):
         # Tests that each header only appears once, regardless of case.
         self.getPage("/headers/doubledheaders")
@@ -752,6 +783,22 @@ llo,
         for key in ['Content-Length', 'Content-Type', 'Date',
                     'Expires', 'Location', 'Server']:
             self.assertEqual(hnames.count(key), 1)
+        
+        if cherrypy.config.get('server.protocol_version') == "HTTP/1.1":
+            # Test RFC-2047-encoded request and response header values
+            c = "=E2=84=ABngstr=C3=B6m"
+            self.getPage("/headers/ifmatch", [('If-Match', '=?utf-8?q?%s?=' % c)])
+            self.assertBody("u'\\u212bngstr\\xf6m'")
+            self.assertHeader("ETag", '=?utf-8?b?4oSrbmdzdHLDtm0=?=')
+            
+            # Test a *LONG* RFC-2047-encoded request and response header value
+            self.getPage("/headers/ifmatch",
+                         [('If-Match', '=?utf-8?q?%s?=' % (c * 10))])
+            self.assertBody("u'%s'" % ('\\u212bngstr\\xf6m' * 10))
+            self.assertHeader("ETag",
+                              '=?utf-8?b?4oSrbmdzdHLDtm3ihKtuZ3N0csO2beKEq25nc3Ryw7Zt4oSrbmdzdHLDtm0=?='
+                              '=?utf-8?b?4oSrbmdzdHLDtm3ihKtuZ3N0csO2beKEq25nc3Ryw7Zt4oSrbmdzdHLDtm0=?='
+                              '=?utf-8?b?4oSrbmdzdHLDtm3ihKtuZ3N0csO2bQ==?=')
     
     def testHTTPMethods(self):
         # Test that all defined HTTP methods work.
@@ -824,20 +871,31 @@ llo,
         self.assertBody(data)
     
     def testCookies(self):
-        self.getPage("/cookies/single?name=First",
-                     [('Cookie', 'First=Dinsdale;')])
-        self.assertHeader('Set-Cookie', 'First=Dinsdale;')
-        
-        self.getPage("/cookies/multiple?names=First&names=Last",
-                     [('Cookie', 'First=Dinsdale; Last=Piranha;'),
-                      ])
-        self.assertHeader('Set-Cookie', 'First=Dinsdale;')
-        self.assertHeader('Set-Cookie', 'Last=Piranha;')
+        import sys
+        if sys.version_info >= (2, 5):
+            self.getPage("/cookies/single?name=First",
+                         [('Cookie', 'First=Dinsdale;')])
+            self.assertHeader('Set-Cookie', 'First=Dinsdale')
+            
+            self.getPage("/cookies/multiple?names=First&names=Last",
+                         [('Cookie', 'First=Dinsdale; Last=Piranha;'),
+                          ])
+            self.assertHeader('Set-Cookie', 'First=Dinsdale')
+            self.assertHeader('Set-Cookie', 'Last=Piranha')
+        else:
+            self.getPage("/cookies/single?name=First",
+                         [('Cookie', 'First=Dinsdale;')])
+            self.assertHeader('Set-Cookie', 'First=Dinsdale;')
+            
+            self.getPage("/cookies/multiple?names=First&names=Last",
+                         [('Cookie', 'First=Dinsdale; Last=Piranha;'),
+                          ])
+            self.assertHeader('Set-Cookie', 'First=Dinsdale;')
+            self.assertHeader('Set-Cookie', 'Last=Piranha;')
     
     def testMaxRequestSize(self):
         self.getPage("/", headers=[('From', "x" * 500)])
         self.assertStatus(413)
-        self.assertInBody("Request Entity Too Large")
         
         # Test for http://www.cherrypy.org/ticket/421
         # (Incorrect border condition in readline of SizeCheckWrapper).
@@ -866,7 +924,6 @@ Content-Type: text/plain
              ("Content-Length", len(b))]
         self.getPage('/upload', h, "POST", b)
         self.assertStatus(413)
-        self.assertInBody("Request Entity Too Large")
     
     def testEmptyThreadlocals(self):
         results = []
